########################################################################
# latex.mk
#
# Make production of LaTeX documents trivial, not just easy.
#
####################################
# Usage.
#
# In order to use this, simply define a variable PRODUCTS to specify
# your documents, like:
#
#   PRODUCTS = mydoc1.pdf mydoc2.pdf
#
# You may specify other variables controlling behavior at this point
# (see Customizable Variables (I)), such as PDFMODE (which defaults to,
# "-pdf"), e.g.:
#
#   PDFMDOE = -xelatex # or -luatex
#
# Then, include this make fragment with something like:
#
####################################
# ifneq (,$(SSDDOC_INC))
#   ldir = $(SSDDOC_INC)
# else
#   ldir = ../cet-is/ssddoc/include
# endif
#
# include $(ldir)/latex.mk
####################################
#
# If the ssddoc UPS product is set up and/or SSDDOC_INC is defined in
# the environment or elsewhere, then $(SSDDOC_INCLUDE) is used to find
# us, otherwise ldir should be set appropriately. The above example
# works if you have the cet-is repository checked out in the parent
# directory of the one in which your GNUmakefile resides.
#
# Following the include of this fragment, you may wish to override other
# customizable variables such as clean_dirs or clean_files (see
# Customizable Variables (II)), e.g.:
#
#   override clean_dirs += more dirs to clean
#
#   override clean_files += more files to clean
#
# See the Customizable Variables sections for variables that may be
# overridden, and what their default values might be, if any.
#
####################################
# Defined targets.
#
# An unqualified "make" is equivalent to, "make all," which will produce
# all defined $(PRODUCTS) unless you define the ALL variable to be
# (e.g.) some subset of $(PRODUCTS), or perhaps "final" (see definition
# of same, below).
#
# Other targets:
#
#   clean:
#     Remove $(clean_dirs) and generated files (according to latexmk).
#
#   clobber:
#     Additionally remove $(products).
#
#   echo_<var>:
#     For debugging: print out the value of <var>, and its origin.
#
#   final:
#     Generate final versions of all PDF files with fonts embedded (see
#     <prod>-final.pdf below for more details).
#
#   macenv:
#     Add the LaTeX environment variables (TEXINPUTS, etc.) to the
#     Mac OS X GUI environment to enable applications such as
#     aquamacs started from the GUI to have the correct settings
#     for (e.g.) internal LaTeX invocation.
#
#   view:
#     Start the correctly defined viewer for each defined $(PRODUCT).
#     e.g. to set the pdf view to skim, you need a file ~/.latexmkrc
#     containing:
#
#       $pdf_previewer = "open -a skim %O %S"
#
#   <prod>:
#     Make the single product <prod> (should be a member of
#     $(PRODUCTS)).
#
#   <prod>-final.pdf:
#     For PDF files only, produce a final version of the product with
#     fonts embedded.
#
#   view-<prod>:
#     Run the defined viewer for <prod> after first making sure that it
#     is up-to-date.
#
####################################
# Notes.
#
# * LATEX_MK_DIR is set to the directory from which this file was
#   included, regardless of how it was found. Use it to set (e.g.)
#   TEXINPUTS, if appropriate.
#
# * If you need to add targets (generation of some dependencies for
#   example, like figures), you should define that target, eg:
#
#     myFigure.tex: myFigure.dat
#
#   The latexmk machinery should take care of ensuring that it is invoked
#   at the correct time.
#
# * If you intend to have BibTeX autogenerate the .bbl file and would
#   like it cleaned up automatically, add "-bibtex" to
#   LATEXMK_EXTRA_OPTS.
#
# * If you are knitting a .tex file from a .Rnw file, it will be treated
#   as an intermediate file and removed after the generation of the .pdf
#   file. In order to keep it around, either make it explicitly (e.g.
#   for debugging purposes) or add it to the .SECONDARY special target
#   in your local GNUmakefile.
#
# * Most targets are not echoed unless VERBOSE is set.
#
# * Error output from clean targets is suppressed unless DEBUG is set.
#
########################################################################

########################################################################
# Customizable Variables (I).
#
# These variables may be specified on the command line or in the
# environment, or overridden following the include of this make
# fragment.
########################################################################

# Contents of ALL target
ifeq (,$(ALL))
  ALL = $(PRODUCTS)
endif

# Default way to generate PDF.
ifeq (,$(PDFMODE))
  PDFMODE = -pdf
endif

# Extra command-line options.
ifeq (,$(LATEXMK_EXTRA_OPTS))
  LATEXMK_EXTRA_OPTS =
endif

# Extra perl directives for LATEXMK.
ifeq (,$(LATEXMK_EXTRA_CMDS))
  LATEXMK_EXTRA_CMDS =
endif

# GhostScript executable.
ifeq (,$(GS))
  GS = gs
endif

# Rscript executable.
ifeq (,$(RSCRIPT))
  RSCRIPT = Rscript
endif

# Echo.
ifeq (,$(ECHO))
  ECHO = @/bin/echo
endif

########################################################################
# Customizable Variables (II).
#
# These variables may be overridden following the include of this make
# fragment.
########################################################################

# Files removed by, "make tidy."
tidy_files =  \
  *.bak \
  *.orig \
  *.rej \
  *.old \
  *~ \
	*~[0-9]

# Extra directories to make sure we empty.
clean_dirs = $(DEPS_DIR)

# Extra files to make sure we remove.
clean_files =

# Variables to push to environment with "make macenv"
tex_env_vars = \
  TEXINPUTS \
  BSTINPUTS \
  BIBINPUTS \
  MFINPUTS \
  MPINPUTS

########################################################################
# No user-serviceable parts below.
########################################################################

####################################
# Define "all" target early.

all: $(ALL)

####################################
# Variable definitions.

DEPS_DIR = .deps

LATEXMK = latexmk -f- \
            -recorder -use-make -deps \
            $(LATEXMK_EXTRA_OPTS) \
            -e 'push @generated_exts, qw(nav snm vrb);' \
            -e '$(LATEXMK_EXTRA_CMDS)'

ensure_deps = mkdir -p $(DEPS_DIR)

safe_fail = { $(RM) "$@"; false; }

def_maybe_pushenv = \
function maybe_pushenv() \
{ \
  local var; \
  for var in "$$@"; do \
    if [[ -n "$$var" ]] && [[ -n "$$(eval echo \"\$$$$var\")" ]] ; then \
      launchctl setenv "$$var" "$$(eval echo \"\$$$$var\")" && \
        echo "Pushed $$var to application env."; \
    else \
      true; \
    fi; \
  done; \
}

find_ourselves = \
  file=($(MAKEFILE_LIST)); \
    while [[ ! -f "$${file[*]}" ]]; do \
      file=("$${file[@]:1}"); \
    done; \
    echo $$(dirname "$${file[*]}")

LATEX_MK_DIR := $(shell $(find_ourselves))

AT = $(if $(VERBOSE),,@)

SILENT = $(if $(DEBUG),,>/dev/null 2>&1)

####################################
# Dependencies.

$(foreach file,$(PRODUCTS),$(eval -include $(DEPS_DIR)/$(file).d))

####################################
# Targets

.PHONY: clean clobber macenv tidy cleanit cleandirs cleanfiles

$(filter %.dvi, $(PRODUCTS)): %.dvi : %.tex
	@$(ensure_deps)
	$(AT)$(LATEXMK) -dvi -deps-out=$(DEPS_DIR)/$@.d $(<) || $(safe_fail)

$(filter %.pdf, $(PRODUCTS)): %.pdf : %.tex
	@$(ensure_deps)
	$(AT)$(LATEXMK) $(PDFMODE) -deps-out=$(DEPS_DIR)/$@.d $(<) || $(safe_fail)

$(filter %.ps, $(PRODUCTS)): %.ps : %.tex
	@$(ensure_deps)
	$(AT)$(LATEXMK) -ps -deps-out=$(DEPS_DIR)/$@.d $(<) || $(safe_fail)

cleanit: cleandirs cleanfiles

cleandirs:
	$(AT)$(RM) -r $(clean_dirs) $(SILENT)

cleanfiles:
	-$(AT)$(LATEXMK) $(clean_arg) -use-make- -deps- -f $(PRODUCTS) $(SILENT)
	$(AT)$(RM) $(clean_files) $(SILENT)

clean: clean_arg = -c
clean: cleanit

tidy:
	$(AT)$(RM) $(tidy_files) $(SILENT)

clobber: clean_arg = -C
clobber: cleanit
	$(AT)$(RM) $(filter %.pdf,$(foreach d,$(PRODUCTS),$(d:.pdf=-final.pdf))) $(SILENT)

macenv:
	@$(def_maybe_pushenv); maybe_pushenv $(tex_env_vars)

# View targets
.PHONY: $(foreach d, $(PRODUCTS), view-$d) view

$(filter %.pdf,$(foreach d,$(PRODUCTS),view-$d)) : view-% : %
	$(AT)latexmk $(PDFMODE) -pv $(*:.pdf=)

$(filter %.dvi,$(foreach d,$(PRODUCTS),view-$d)) : view-% : %
	$(AT)latexmk -dvi -pv $(*:.dvi=)

$(filter %.ps,$(foreach d,$(PRODUCTS),view-$d))  : view-% : %
	$(AT)latexmk -ps -pv $(*:.ps=)

view: $(foreach v,$(filter %.ps %.dvi %.pdf,$(PRODUCTS)),view-$(v))

# Embed fonts for final version.
$(filter %.pdf,$(foreach d,$(PRODUCTS),$(d:.pdf=-final.pdf))) : %-final.pdf : %.pdf
	$(AT)$(GS) -dNOPAUSE -dBATCH -sDEVICE=pdfwrite -dPDFSETTINGS=/prepress -dEmbedAllFonts=true -sOutputFile=$(@) -f $(<)

.PHONY: final

final: $(filter %.pdf,$(foreach d,$(PRODUCTS),$(d:.pdf=-final.pdf)))

# Knit:
.INTERMEDIATE: $(foreach d,$(wildcard *.Rnw),$(d:.Rnw=.tex))

%.tex : %.Rnw
	$(AT)$(RSCRIPT) -e "knitr::knit('$(<)')"

# Debugging
echo_%:
	$(ECHO) "$(subst echo_,,$@)=\"$($(subst echo_,,$@))\""
	$(ECHO) "origin $(subst echo_,,$@) returns $(origin $(subst echo_,,$@))"

### Local Variables:
### mode: makefile-gmake
### End:
